#include "PhysicalMemory.h"

namespace HLE::Libs::LibKernel::MemoryManagement {



void PhysicalMemoryInit() { g_physical_memory = new PhysicalMemory; }

static  u64 align_pos(u64 pos, u64 align) { return (align != 0 ? (pos + (align - 1)) & ~(align - 1) : pos); }

bool PhysicalMemory::Alloc(u64 searchStart, u64 searchEnd, u64 len, u64 alignment, u64* physAddrOut, int memoryType) {
    
    u64 find_free_pos = 0;

    //iterate through allocated blocked and find the next free position
    if (!m_allocatedBlocks.empty()) {
        for (const auto& b : m_allocatedBlocks) {
            u64 n = b.start_addr + b.size;
            if (n > find_free_pos) {
                find_free_pos = n;
            }
        }
    }
    
    //align free position
    find_free_pos = align_pos(find_free_pos, alignment);

    //if the new position is between searchStart - searchEnd , allocate a new block
    if (find_free_pos >= searchStart && find_free_pos + len <= searchEnd) {
        AllocatedBlock b{};
        b.size = len;
        b.start_addr = find_free_pos;
        b.memoryType = memoryType;

        m_allocatedBlocks.push_back(b);

        *physAddrOut = find_free_pos;
        return true;
    }

    return false;
}

}  // namespace HLE::Libs::LibKernel::MemoryManagement